@@ -5,7 +5,7 @@
  */
 #define DETECT_WRAPPED_SVC
 #define SMART_TCP_RW
-#undef PRINT_UNKOWN_SVC_BANNER
+
 
 #include <includes.h>
 
@@ -62,7 +62,7 @@ int plugin_init(desc)
  struct arglist * desc;
 { 
  plug_set_id(desc, 10330);
- plug_set_version(desc, "$Revision: 1.183 $");
+ plug_set_version(desc, "$Revision: 1.200 $");
  
  plug_set_name(desc, FR_NAME, "francais");
  plug_set_name(desc, EN_NAME, NULL);
@@ -98,9 +98,6 @@ int plugin_init(desc)
 #define TEST_SSL_PREF	"Test SSL based services"
  add_plugin_preference(desc, TEST_SSL_PREF, PREF_RADIO, "All;Known SSL ports;None");
 #endif
-#define QUICK_SOCKS_PREF	"Quick SOCKS proxy checking"
- add_plugin_preference(desc, QUICK_SOCKS_PREF, PREF_CHECKBOX, "yes");
- 
  plug_set_timeout(desc, PLUGIN_TIMEOUT*4);
  return(0);
 }
@@ -1431,8 +1428,11 @@ mark_unknown_svc(desc, port, banner, trp)
     case 901:	norm = "SWAT"; break;
     case 993:	norm = "IMAPS"; break;
     case 995:	norm = "POP3S"; break;
+#if 0
     case 1080:	norm = "SOCKS"; break;
+#endif
     case 1109:	norm = "KPOP"; break; /* ? */
+    case 2309:  norm = "Compaq Management Server"; break;
     case 2401:	norm = "CVSpserver"; break;
     case 3128:	norm = "Squid"; break;
     case 3306:	norm = "MySQL"; break;
@@ -1448,51 +1448,6 @@ It is usually reserved for %s",
 	      get_encaps_through(trp), norm);
     }
 
-#ifdef PRINT_UNKNOWN_SVC_BANNER
-  if (banner != NULL && *banner != '\0')
-    {
-      char	*p;
-      int	i, j, n;
-
-      if (*tmp == '\0')
-	snprintf(tmp, sizeof(tmp), "An unknown service is running on this port%s.\n",
-		get_encaps_through(trp));
-
-      for (p = tmp; *p != '\0'; p ++)
-	;
-      n = sprintf(p, "\nHere is its%s banner:",
-		  strlen(banner) > 256 ? " (truncated)" : "");
-      if (n > 0)
-	p += n;
-      
-      for (j = 0; j < 256; j += 16)
-	{
-	  *p++ = '\n';
-	  for (i = j; i < j + 16 && banner[i] != '\0'; i ++)
-	    {
-	      sprintf(p, "%02x ", banner[i]);
-	      p += 3;
-	    }
-	  /* Fill last line */
-	  for (; i < j + 16; i ++)
-	    {
-	      *p++ = ' '; *p++ = ' '; *p++ = ' '; 
-	    }
-
-	  *p++ = ' '; *p++ = ' ';
-	  for (i = j; i < j + 16 && banner[i] != '\0'; i ++)
-	    if (isspace(banner[i]))
-	      *p++ = ' ';
-	    else if (isprint(banner[i]))
-	      *p++ = banner[i];
-	    else
-	      *p++ = '.';
-	  if (banner[i] == '\0')
-	    break;
-	}
-      *p++ = '\0';
-    }
-#endif
   if (*tmp != '\0')
     post_note(desc, port, tmp);
 }
@@ -1571,7 +1526,8 @@ mark_zebra_server(desc, port, buffer, trp)
   char ban[512];
 
  register_service(desc, port, "zebra");
- 
+ snprintf(ban, sizeof(ban), "zebra/banner/%d", port);
+ plug_set_key(desc, ban, ARG_STRING, buffer);
  snprintf(ban, sizeof(ban), "A zebra daemon (bgpd or zebrad) is running on this port%s",
 	     get_encaps_through(trp));
  post_note(desc, port, ban);	 
@@ -1687,6 +1643,21 @@ struct arglist * desc;
  post_note(desc, port, ban);	 
 }
 
+void
+mark_jabber_server(desc, port, buffer, trp)
+ struct arglist * desc;
+ int port, trp;
+ char * buffer;
+{
+ char ban[255];
+ register_service(desc, port, "jabber");
+ snprintf(ban, sizeof(ban), "jabber daemon seems to be running on this port%s",
+   get_encaps_through(trp));
+ post_note(desc, port, ban);
+}
+
+
+
 static void
 mark_avotus_mm_server(desc, port, buffer, trp)
 struct arglist * desc;
@@ -1703,187 +1674,28 @@ struct arglist * desc;
 }
 
 static void
-mark_socks_proxy(desc, port, ver, pa, authm)
+mark_socks_proxy(desc, port, ver)
      struct arglist	* desc;
      int		port, ver;
-     unsigned char	*pa;
-     int		authm;
 {
-  char			str[256], s[80], ipstr[16], *p;
-  int			n;
-  struct in_addr	*t = plug_get_host_ip(desc);
-
-#ifdef DEBUG
-  fprintf(stderr, "mark_socks_proxy: port=%d ver=%d authm=%d\n",
-	  port, ver, authm);
-#endif
-  
-  snprintf(str, sizeof(str), "socks%d", ver); register_service(desc, port, str);
-
-  n = snprintf(str, sizeof(str), "A SOCKS%d proxy is running on this port. ", ver);
-  p = str + n;
-
-  if (pa == NULL)
-    strcpy(p, ";We could not determine its external interface address");
-  else if (t == NULL || memcmp(t, pa, 4) != 0)
-    {
-      snprintf(ipstr, sizeof(ipstr), "%d.%d.%d.%d",
-	      (int)pa[0], (int)pa[1], (int)pa[2], (int)pa[3]);
-      sprintf(p, ";Its external interface address is %s", ipstr);
-      snprintf(s, sizeof(s), "socks%d/ext_ip/%d", ver, port);
-      plug_set_key(desc, s, ARG_STRING, ipstr);
-    }
-  if (ver > 4 && authm != -1)
-    {
-      snprintf(s, sizeof(s), "socks%d/auth/%d", ver, port);
-      plug_set_key(desc, s, ARG_INT, (void*) authm);
-
-      while (*p != '\0')
-	p++;
-      switch (authm)
-	{
-	case 0:
-	  strcpy(p, ";It does not require authentication, or does not implement it.");
-	  break;
-	case 1:
-	  strcpy(p, ";It prefers the username/password authentication.");
-	  break;
-	case 2:
-	  strcpy(p, ";It prefers the GSS API authentication");
-	  break;
-	case 0xFF:
-	  strcpy(p, ";It rejected all standard authentication methods (none, password, GSS API)");
-	  break;
-	default:
-	  /* Should *not* happen: the SOCKS proxy chosed a method that we 
-	   * did not propose */
-	  sprintf(p, "; It prefers the 0x%x authentication method (bug?)",
-		 authm);
-	  break;
-	}
-    }
+  char			str[256];
 
+  sprintf(str, "socks%d", ver);
+  register_service(desc,port, str);
+  sprintf(str, "A SOCKS%d proxy is running on this port. ", ver);
   post_note(desc, port, str);
 }
 
-static int
-check_for_socks_proxy(desc, port, trp)
-     struct arglist	*desc;
+static void
+mark_direct_connect_hub(desc, port, trp)
+     struct arglist	* desc;
      int		port, trp;
 {
-  int		cnx, len, found = 0;
-  unsigned char	buf[64];
-  char		*p;
-  int		authm = -1;
+  char			str[256];
 
-#ifdef DEBUG
-  fprintf(stderr, "check_for_socks_proxy: port=%d\n", port);
-#endif
-  if ((cnx = open_stream_connection(desc, port, trp, 2)) >= 0)
-    {
-      /*
-       * SOCKS4 request: 
-       * 1	Version number (4)
-       * 1	Command (1: connect / 2: bind)
-       * 2	Port
-       * 4	Address
-       * Var	UserID
-       * 1	zero (0)
-       */
-#if 0
-      /* connect: 127.0.0.1:65535 */
-#define SOCKS4_REQ "\004\001\xFF\xFF\x7F\0\0\001root\0"
-#else
-      /* Bind: (local) port = 65535; expected remote address = 10.10.10.10 */
-#define SOCKS4_REQ "\004\002\xFF\xFF\x0A\x0A\x0A\x0Aroot\0"
-#endif
-
-      write_stream_connection(cnx, SOCKS4_REQ, sizeof(SOCKS4_REQ)-1);
-      len = read_stream_connection_min(cnx, buf, 8, 8);
-      if (len == 8)
-	{
-	  /*
-	   * SOCKS4 answer:
-	   * 1	version (0)
-	   * 1	code (90 -> 92)
-	   * 2	port (or 0)
-	   * 4	IP (or 0)
-	   */
-	  if (buf[0] == 0 && buf[1] >= 90 && buf[1] <= 93)
-	    {
-	      /* really looks like a SOCKS4 server */
-	      if (buf[1] == 90)
-		p = buf+4;
-	      else
-		p = NULL;
-	      mark_socks_proxy(desc, port, 4, p, authm);
-	      found ++;
-	    }
-	}
-      close_stream_connection(cnx);
-    }
-
-  if ((cnx = open_stream_connection(desc, port, trp, 2)) >= 0)
-    {
-      /*
-       * SOCKS5 connection: 
-       * 1	Version number (5)
-       * 1	# of auth methods 
-       * Var	Array of methods:
-       *	1	Method number:	0: no auth
-       *				1: GSSAPI
-       *				2: password 
-       *				3-7F: IANA reserved,
-       *				80-FE: user reserved
-       *				FF: no method 
-       *
-       * We should announce at least GSS API to be RFC conformant.
-       *
-       * The server answers:
-       * 1	Version
-       * 1	Chosen method (or FF if failure)
-       */
-#if 1
-# define SOCKS5_CNX "\005\003\0\1\2"
-#else
-# define SOCKS5_CNX "\005\001\0"
-#endif
-      write_stream_connection(cnx, SOCKS5_CNX, sizeof(SOCKS5_CNX)-1);
-      len = read_stream_connection_min(cnx, buf, 2, 2);
-      if (len == 2)
-	{
-#if 0
-	  fprintf(stderr, "Buf = %02x %02x %02x %02x\n",
-		  buf[0], buf[1], buf[2], buf[3]);
-#endif
-	  if (buf[0] == 5 && (buf[1] <= 2 || buf[1] == 0xFF))
-	    /* buf[1] >= 0 is always true! */
-	    {
-	      authm = buf[1];
-	      /*
-	       * really looks like a SOCKS5 server
-	       */
-#define SOCKS5_REQ "\005\002\0\001\x0A\x0A\x0A\x0A\xFF\xFF" /* bind */
-	      write_stream_connection(cnx, SOCKS5_REQ, sizeof(SOCKS5_REQ)-1);
-	      len = read_stream_connection_min(cnx, buf, 10, 10);
-	      if (buf[1] != 0 || buf[3] != 1)
-		{
-#if 0
-		  fprintf(stderr, "Buf = %02x %02x %02x %02x\n",
-			  buf[0], buf[1], buf[2], buf[3]);
-#endif	  
-		  p = NULL;
-		}
-	      else
-		p = buf + 4;
-	      mark_socks_proxy(desc, port, 5, p, authm);
-	      found ++;
-	    }
-	}
-      close_stream_connection(cnx);
-    }
-
-  return found;
+  register_service(desc,port, "DirectConnectHub");
+  snprintf(str, sizeof(str), "A Direct Connect Hub is running on this port%s", get_encaps_through(trp));
+  post_note(desc, port, str);
 }
 
 /*
@@ -1944,6 +1756,7 @@ known_ssl_port(int port)
     case 994:			/* IRCS */
     case 995:			/* POP3S */
     case 1241:			/* Nessus */
+    case 2381:			/* Compaq Web Management (HTTPS) */
     case 2478:			/* SecurSight Authentication Server (SSL) */
     case 2479:			/* SecurSight Event Logging Server (SSL) */
     case 2482:			/* Oracle GIOP SSL */
@@ -2011,16 +1824,12 @@ static int plugin_do_run(desc, h, test_ssl)
  char	*wrap_timeout_s = get_plugin_preference(desc, WRAP_TIMEOUT_PREF);
 #endif
   unsigned char	*p;
-  char	*quick_socks_s = get_plugin_preference(desc, QUICK_SOCKS_PREF);
   time_t	t1, t2;
   int		delta_t;
   fd_set	rfds, wfds, xfds;
   struct timeval tv;
   
 
-  if (quick_socks_s != NULL && strcmp(quick_socks_s, "no") == 0)
-    might_be_socks = 1;
-
   if (rw_timeout_s != NULL && (x = atoi(rw_timeout_s)) > 0)
     rw_timeout = x;
   if (cnx_timeout_s != NULL && (x = atoi(cnx_timeout_s)) > 0)
@@ -2089,6 +1898,7 @@ static int plugin_do_run(desc, h, test_ssl)
 #ifdef SMART_TCP_RW
     if (trp == NESSUS_ENCAPS_IP && realfd >= 0)
       {
+select_again:      
 	FD_ZERO(&rfds); FD_ZERO(&wfds);
 	FD_SET(realfd, &rfds); FD_SET(realfd, &wfds); 
 
@@ -2096,7 +1906,10 @@ static int plugin_do_run(desc, h, test_ssl)
 	tv.tv_usec = 0; tv.tv_sec = rw_timeout;
 	x = select(realfd+1, &rfds, &wfds, NULL, &tv);
 	if (x < 0)
-	  perror("select");
+          {
+           if ( errno == EINTR ) goto select_again;
+	   perror("select");
+          }
 	else if (x == 0)
 	  timeout = 1;
 	else if (x > 0)
@@ -2123,16 +1936,16 @@ static int plugin_do_run(desc, h, test_ssl)
    if (len > 0)
      {
      banner = estrdup(buffer);
-   for(i=0;i<len;i++)buffer[i] = tolower(buffer[i]);      
-   line = estrdup(buffer);
+     for(i=0;i<len;i++)buffer[i] = tolower(buffer[i]);      
+     line = estrdup(buffer);
   
-   if(strchr(line, '\n')){
+   if( strchr(line, '\n') != NULL ){
    	char * t = strchr(line, '\n');
 	t[0]='\0';
 	}
 	
    origline = estrdup(banner);
-   if(strchr(origline, '\n')){
+   if( strchr(origline, '\n') != NULL ){
    	char * t = strchr(origline, '\n');
 	t[0] = '\0';
 	}
@@ -2169,6 +1982,8 @@ static int plugin_do_run(desc, h, test_ssl)
     else if(strstr(banner, "Eggdrop") != NULL &&
             strstr(banner, "Eggheads") != NULL )
       mark_eggdrop_server(desc, port, origline, trp);	    
+    else if(strncmp(line, "$lock ", strlen("$lock ")) == 0 )
+	mark_direct_connect_hub(desc, port, trp);
     else if(len > 34 && strstr(&(buffer[34]), "iss ecnra"))
     	mark_iss_realsecure(desc, port, origline, trp); 
     else if(len == 4 && origline[0] == 'Q' && origline[1] == 0 && origline[2] == 0 && origline[3] == 0)
@@ -2177,12 +1992,12 @@ static int plugin_do_run(desc, h, test_ssl)
         mark_adsgone(desc, port, origline, trp);
     else if(strncmp(line, "icy 200 ok", strlen("icy 200 ok")) == 0)
         mark_shoutcast_server(desc, port, origline, trp);
-    else  if((strstr(line, "smtp") || strstr(line, "simple mail transfer") || strstr(line, "mail server") || strstr(line, "messaging")) && !strncmp(line, "220", 3))
+    else  if((strstr(line, "smtp") || strstr(line, "simple mail transfer") || strstr(line, "mail server") || strstr(line, "messaging") || strstr(line, "Weasel")) && !strncmp(line, "220", 3))
     	mark_smtp_server(desc, port, origline, trp);
     else if(strstr(line, "220 ***************")) /* CISCO SMTP (?) - see bug #175 */
         mark_smtp_server(desc, port, origline, trp);
     else if(strstr(line, "220 esafealert") != NULL )
-    	mark_smtp_server(desc, port, origline, trp);	
+    	mark_smtp_server(desc, port, origline, trp);
     else if(strncmp(line, "220", 3) == 0 &&
 	    strstr(line, "groupwise internet agent") != NULL)
       	mark_smtp_server(desc, port, origline, trp);
@@ -2191,7 +2006,9 @@ static int plugin_do_run(desc, h, test_ssl)
      else if(strncmp(line, "200", 3) == 0 &&
       	    strstr(line, "mail ") != NULL)
 	mark_smtp_server(desc, port, origline, trp);
-    else if((!strncmp(line, "hhost '", 7) || !strncmp(line, "khost '", 7) || !strncmp(line, "whost '", 7)) && strstr(line, "mysql"))
+     else if (strncmp(line, "421", 3) == 0 && strstr(line, "smtp ") != NULL)
+       mark_smtp_server(desc, port, origline, trp);
+    else if( (strncmp(line + 1, "host '", 6) == 0 ) && strstr(line, "mysql") != NULL )
         mark_mysql(desc, port, origline, trp);
     else if(!strncmp(line, "efatal", 6) || !strncmp(line, "einvalid packet length", strlen("einvalid packet length")))
         mark_postgresql(desc, port, origline, trp);
@@ -2224,6 +2041,8 @@ static int plugin_do_run(desc, h, test_ssl)
     	mark_imap_server(desc, port, origline, trp);
     else if(strstr(line, "*ok iplanet messaging multiplexor"))
         mark_imap_server(desc, port, origline, trp);
+    else if(strstr(line, "*ok communigate pro imap server"))
+        mark_imap_server(desc, port, origline, trp);
     else if(strstr(line, "* ok courier-imap"))
         mark_imap_server(desc, port, origline, trp);
     else if(strncmp(line, "giop", 4) == 0)
@@ -2319,7 +2138,7 @@ static int plugin_do_run(desc, h, test_ssl)
     	 mark_LISa_server(desc, port, banner, trp);
     else if(strlen(banner) == 3 && banner[2] == '\n')
     	 mark_msdtc_server(desc, port, banner, trp);
-    else if((!strncmp(line, "220", 3) && strstr(line, "poppassd")) || !strncmp(line, "200", 3))
+    else if((!strncmp(line, "220", 3) && strstr(line, "poppassd")))
     	mark_pop3pw_server(desc, port, origline, trp);
     else if(strstr(line, "welcome!psybnc@") != NULL)
     	mark_psybnc(desc, port, origline, trp);
@@ -2375,6 +2194,9 @@ static int plugin_do_run(desc, h, test_ssl)
    	   mark_pbmaster_server(desc, port, origline, trp);
    else if(strncmp(line, "pblocald", strlen("pblocald")) == 0)
    	  mark_pblocald_server(desc, port, origline, trp);
+   else if(strncmp(line, "<stream:error>invalid xml</stream:error>",
+      strlen("<stream:error>invalid xml</stream:error>")) ==  0)
+      mark_jabber_server(desc, port, origline, trp);
    else if(strncmp(line, "/c -2 get ctgetoptions", strlen("/c -2 get ctgetoptions")) == 0)
 	  mark_avotus_mm_server(desc, port, origline, trp);
    else if(strncmp(line, "error:wrong password", strlen("error:wrong password")) == 0)
@@ -2406,16 +2228,14 @@ static int plugin_do_run(desc, h, test_ssl)
 	
 	if(p != NULL)*p = c;
       }
+    else if (banner[0] == 5 && banner[1] <= 8 && 
+	     banner[2] == 0 && banner[3] <= 4)
+      mark_socks_proxy(desc, port, 5);
+    else if (banner[0] == 0 && banner[1] >= 90 && banner[1] <= 93)
+      mark_socks_proxy(desc, port, 4);
     else
-      {
-	if (banner[0] == 5 && banner[1] <= 8 && 
-	    banner[2] == 0 && banner[3] <= 4
-	    || 
-	    banner[0] == 0 && (banner[1] >= 90 && banner[1] <= 93))
-	  might_be_socks = 1;
-	unindentified_service = ! flg;
-      }
-   efree(&line);
+      unindentified_service = ! flg;
+    efree(&line);
    efree(&origline); 
    } /* len >= 0 */
    else
@@ -2457,6 +2277,7 @@ static int plugin_do_run(desc, h, test_ssl)
 #if 0
 	   fprintf(stderr, "open_stream_connection(port=%d) succeeded\n", port);
 #endif
+select_again2:
 	   FD_ZERO(&rfds); FD_ZERO(&xfds);
 	   FD_SET(fd, &rfds); FD_SET(fd, &xfds);
 	   tv.tv_sec = wrap_timeout; tv.tv_usec = 0;
@@ -2474,9 +2295,13 @@ static int plugin_do_run(desc, h, test_ssl)
 		   port, x, t2 - t1, wrap_timeout);
 #endif
 	   if (x < 0)
-	     perror("select");
+             {
+              if ( errno == EINTR ) goto select_again2;
+	      perror("select");
+              }
 	   else if (x > 0)
 	     {
+recv_again:             
 	       errno = 0;
 #ifdef MSG_DONTWAIT	       
 	       x = recv(fd, &b, 1, MSG_DONTWAIT);
@@ -2488,7 +2313,10 @@ static int plugin_do_run(desc, h, test_ssl)
 	       fprintf(stderr, "recv(port=%d)=%d\n", port, x);
 
 	       if (x < 0)
+                {
+                 if ( errno == EINTR ) goto recv_again;
 		 perror("recv");
+                }
 		 
 #endif
 
@@ -2538,10 +2366,6 @@ and in %d when we just wait. It is  probably not wrapped\n",
      }
 #endif
 
-   if (might_be_socks && unindentified_service && 
-       check_for_socks_proxy(desc, port, trp) > 0)
-       unindentified_service = 0;
-
    if (unindentified_service && port != 139)
      /* port 139 can't be marked as 'unknown' */
     {
@@ -2615,9 +2439,12 @@ static int fwd_data(int in, int out, pid_t sender)
      else
       m += e;
     }
-   }  
+   }
  }
- else break;
+ else {
+    if ( errno == EINTR )errno = 0;
+    else break;
+    }
  }
  return 0; 
 }
@@ -2763,7 +2590,11 @@ int plugin_run(desc)
   usleep(5000);
   if( sons_args[i] != NULL )
   { 
-   socketpair(AF_UNIX, SOCK_STREAM, 0, sons_pipe[i]);
+   if ( socketpair(AF_UNIX, SOCK_STREAM, 0, sons_pipe[i]) < 0 )
+   {
+    perror("socketpair ");
+    break;
+   }
    sons[i] = fork();
    if(sons[i] == 0)
    {
@@ -2785,8 +2616,11 @@ int plugin_run(desc)
  {
   int flag = 0;
   fd_set rd;
-  struct timeval tv = {0, 5000};
+  struct timeval tv;
   int max = -1;
+  int e;
+  
+  
   FD_ZERO(&rd);
   for (i = 0; i < num_sons ; i ++ )
   { 
@@ -2797,7 +2631,13 @@ int plugin_run(desc)
    }
   }
   
-  if(select(max + 1, &rd, NULL, NULL, &tv))
+again:
+  tv.tv_usec = 5000;
+  tv.tv_sec  = 0;
+  e = select(max + 1, &rd, NULL, NULL, &tv);
+  if ( e < 0 && errno == EINTR ) goto again;
+
+  if(e > 0)
   {
    for( i = 0; i< num_sons ; i ++ )
    {
