@@ -2,7 +2,7 @@
  * nmap_tcp_connect plugin
  *
  * This code is mostly (c) Fyodor <fyodor@dhp.com>. Several changes
- * have been made by Renaud Deraison <deraison@worldnet.fr>.
+ * have been made by Renaud Deraison <deraison@cvs.nessus.org>
  *
  * This plugin is distributed under the GPL
  *
@@ -45,7 +45,7 @@ Facteur de risque : Aucun"
 PlugExport int plugin_init(struct arglist * desc)
 {
  plug_set_id(desc, 10335);
- plug_set_version(desc, "$Revision: 1.18 $");
+ plug_set_version(desc, "$Revision: 1.20 $");
    
          
  plug_set_name(desc, FR_NAME, "francais");
@@ -83,6 +83,7 @@ PlugExport int plugin_run(struct arglist * env)
   int num;
   
   ports = getpts(port_range, &num);
+  if ( ports == NULL ) return 0;
   p_addr = arg_get_value(hostinfos, "IP");
   if( p_addr == NULL )
   	return -1;
